<?php
namespace Teknisa\CADFIS\Strategies;

use Teknisa\Libs\Util\Environment;

class GeneralStrategy
{
    protected $errors = array();
    
    public function __construct(Environment $environment)
    {
		$this->environment = $environment;
	}
	
	protected function setError($rowNum, $columnName, $value, $message)
	{
        array_push(
            $this->errors, 
            array(
                "IDREF" => $rowNum,
                "COLUMN" => $columnName,
                "VALUE" => $value,
                "ERROR_CODE" => $message,
                "DESCERROR" => ""
            )
        );
    }
    
    protected function clearErrors()
	{
	    $this->errors = [];
	}
	
	protected function verifyNumericos($row, $rowNum, $column)
    {
        if(!is_null($row[$column])) {
            if (!ctype_digit(strval($row[$column]))) {
                if (strval($row[$column]) < 0) {
                    self::setError($rowNum, $column, $row[$column], "MUST_BE_GREATER_THAN_EQUAL_ZERO");
                } else {
                    self::setError($rowNum, $column, $row[$column], "ONLY_ACCEPTS_NUMBERS");
                }
            }
        }
    }
    
    protected function verifyGreaterThanZero($row, $rowNum, $column)
    {
        if(!is_null($row[$column])) {
            if (strval($row[$column]) <= 0) {
                self::setError($rowNum, $column, $row[$column], "MUST_BE_GREATER_THAN_ZERO");
            }
        }
    }
    
    protected function verifyLessThan100($row, $rowNum, $column)
    {
        if(!is_null($row[$column])) {
            if (strval($row[$column]) > 100) {
                self::setError($rowNum, $column, $row[$column], "MUST_BE_LESS_THAN_100");
            }
        }
    }
    
    protected function verifyIsEmpty($row, $rowNum, $values)
    {
        foreach ($values as $value) {
            if (empty($row[$value])) {
                self::setError($rowNum, $value, $row[$value], "UNINFORMED");
            } 
        }
    }
    
    protected function verifyCdMarca(&$row, $rowNum)
    {
        if(!is_null($row['CDMARCA'])) {
            
            if (!ctype_digit(strval($row['CDMARCA']))) {
                self::setError($rowNum, 'CDMARCA', $row['CDMARCA'], "ONLY_ACCEPTS_NUMBERS");
            }
            
            if (strlen($row['CDMARCA']) > 5) {
                self::setError($rowNum, 'CDMARCA', $row['CDMARCA'], "5_NUMBERS_EXECEDED");
            }
            
            $row['CDMARCA'] = self::formatColumn($row['CDMARCA'], 5);
        }
    }
    
    protected function verifyIdTpIJurForn($row, $rowNum)
    {
        $columnName = 'IDTPIJURFORN';
        
        if(!is_null($row[$columnName])) {
            if (strlen($row[$columnName]) > 1) {
                self::setError($rowNum, $columnName, $row[$columnName], "1_CHAR_EXECEDED");
            }
    
            if (!strpos('_F_J_O_I_',$row[$columnName])) {
                self::setError($rowNum, $columnName, $row[$columnName], "ONLY_ACCEPTS_F_J_O_I");
            }
        }
    }
    
    protected function concatCdForneced(&$row)
    {
        $row['CDFORNECED'] = $row['IDTPIJURFORN'].$row['NRINSJURFORN'];
    }
    
    private function is10or11($value)
    {
        return (($value == 10) || ($value == 11)) ? 0 : $value;
    }

    private function verifyCPF($cpf) 
    {
        $cpf = str_pad(preg_replace( '/[^0-9]/is', '', $cpf ), 11, '0', STR_PAD_LEFT);
      
        //Verifica o tamanho
        if (strlen($cpf) != 11) {
            return false;
        }
    
        //Verifica se sequencia de numeros iguais Ex: 000.000.000-00
        if (preg_match('/(\d)\1{10}/', $cpf)) {
            return false;
        }
    
        //Verifica primeiro digito
        $soma = 0;
        foreach (str_split(substr($cpf, 0, 9)) as $key => $value) {
            $soma += intval($value) * (10 - $key);
        }
        if(self::is10or11(($soma * 10) % 11) != substr($cpf, 9, 1))
            return false;
        
        //Verifica segundo digito
        $soma = 0;
        foreach (str_split(substr($cpf, 0, 10)) as $key => $value) {
            $soma += intval($value) * (11 - $key);
        }
        if(self::is10or11(($soma * 10) % 11) != substr($cpf, 10, 1))
            return false;
    
        return true;
    }

    private function verifyCNPJ($cnpj) 
    {
        $cnpj = str_pad(preg_replace( '/[^0-9]/is', '', $cnpj ), 14, '0', STR_PAD_LEFT);
        
        // Verifica se o numero de digitos informados é igual a 14 
        if (strlen($cnpj) != 14) {
            return false;
        }
        
        //Verifica se sequencia de numeros iguais Ex: 00.000.000/0000-00
        if (preg_match('/(\d)\1{13}/', $cnpj)) {
            return false;
        }
    
        //Verifica primeiro digito
        $peso = 5;
        $soma = 0;
        foreach (str_split(substr($cnpj, 0, 12)) as $key => $value) {
            $soma += intval($value) * $peso;
            $peso = $peso == 2 ? 9 : $peso - 1;
        }
        if(intval(($soma % 11) < 2 ? 0 : (11 - $soma % 11)) != intval(substr($cnpj, 12, 1)))
            return false;
            
        //Verifica segundo digito
        $peso = 6;
        $soma = 0;
        foreach (str_split(substr($cnpj, 0, 13)) as $key => $value) {
            $soma += intval($value) * $peso;
            $peso = $peso == 2 ? 9 : $peso - 1;
        }
        
        if(intval(($soma % 11) < 2 ? 0 : (11 - $soma % 11)) != intval(substr($cnpj, 13, 1)))
            return false;
    
        return true;
    }
    
    protected function verifyNrInsJurForn(&$row, $rowNum)
    {
        if(!is_null($row['NRINSJURFORN'])) {
            if((!ctype_digit(strval($row['NRINSJURFORN'])))&&($row['IDTPIJURFORN'] == 'J')||($row['IDTPIJURFORN'] == 'F')) {
                self::setError($rowNum, 'NRINSJURFORN', $row['NRINSJURFORN'], "ONLY_ACCEPTS_NUMBERS");
            }
            
            if (strlen($row['NRINSJURFORN']) > 20) {
                self::setError($rowNum, 'NRINSJURFORN', $row['NRINSJURFORN'], "20_NUMBERS_EXECEDED");
            } else {
                
                if($row['IDTPIJURFORN'] == 'J') {
                    //CNPJ
                    if (strlen($row['NRINSJURFORN']) > 14) {
                        self::setError($rowNum, 'NRINSJURFORN', $row['NRINSJURFORN'], "MAX_IDTPIJURFORN_J");
                    } else {
                        
                        $row['NRINSJURFORN'] = self::formatColumn($row['NRINSJURFORN'], 14, true);    
                        
                        if(!self::verifyCNPJ($row['NRINSJURFORN'])) {
                            self::setError($rowNum, 'NRINSJURFORN', $row['NRINSJURFORN'], "INVALID_CNPJ");
                        }
                    }
                    
                } else if($row['IDTPIJURFORN'] == 'F') {
                    //CPF
                    if (strlen($row['NRINSJURFORN']) > 11) {
                        self::setError($rowNum, 'NRINSJURFORN', $row['NRINSJURFORN'], "MAX_IDTPIJURFORN_F");
                    } else {
                        
                        $row['NRINSJURFORN'] = self::formatColumn($row['NRINSJURFORN'], 11, true);
                        
                        if(!self::verifyCPF($row['NRINSJURFORN'])) {
                            self::setError($rowNum, 'NRINSJURFORN', $row['NRINSJURFORN'], "INVALID_CPF");
                        }
                    }
                    
                } else if($row['IDTPIJURFORN'] == 'O') {
                    //Outros
                    if (strlen($row['NRINSJURFORN']) > 5) {
                        self::setError($rowNum, 'NRINSJURFORN', $row['NRINSJURFORN'], "MAX_IDTPIJURFORN_O");
                    } else {
                        $row['NRINSJURFORN'] = self::formatColumn($row['NRINSJURFORN'], 5);
                    }
                } else {
                    //Internacional
                    $row['NRINSJURFORN'] = self::formatColumn($row['NRINSJURFORN'], 20);
                }
            }
        }
    }
    
    protected function verifyCdProduto(&$row, $rowNum)
    {
        if(!is_null($row['CDARVPROD'])) {
            if (strlen($row['CDARVPROD']) > 20) {
                self::setError($rowNum, 'CDARVPROD', $row['CDARVPROD'], "20_CHAR_EXECEDED");
            }
            
            $cdproduto = $this->service->verifyCdProdutoExists($row);
            if (!$cdproduto) {
                self::setError($rowNum, 'CDARVPROD', $row['CDARVPROD'], "UNREGISTERED");
            }
        }
    }
    
    protected function verifyUnidade($row, $rowNum)
    {
        $columnName = 'SGUNIDADE';
        
        if(!is_null($row[$columnName])) {
            if (!$this->service->verifyUnidadeExists($row)) {
                self::setError($rowNum, $columnName, $row[$columnName], "UNREGISTERED");
            }
        }
    }
    
    protected function verifyCdRegiao(&$row, $rowNum)
    {
        if(!is_null($row['CDREGIAO'])) {
            $row['CDREGIAO'] = self::formatColumn($row['CDREGIAO'], 3);
            
            if (!ctype_digit(strval($row['CDREGIAO']))) {
                self::setError($rowNum, 'CDREGIAO', $row['CDREGIAO'], "ONLY_ACCEPTS_NUMBERS");
            }
            
            if (strlen($row['CDREGIAO']) > 3) {
                self::setError($rowNum, 'CDREGIAO', $row['CDREGIAO'], "3_NUMBERS_EXECEDED");
            }
        }
        
    }
    
    protected function verifyCdPais(&$row, $rowNum, $columnName = 'CDPAIS')
    {
        if(!is_null($row[$columnName])) {
            
            if (!ctype_digit(strval($row[$columnName]))) {
                self::setError($rowNum, $columnName, $row[$columnName], "ONLY_ACCEPTS_NUMBERS");
            }
            
            if (strlen($row[$columnName]) > 4) {
                self::setError($rowNum, $columnName, $row[$columnName], "4_NUMBERS_EXECEDED");
            }
            
            $row[$columnName] = self::formatColumn($row[$columnName], 4);
        }
    }
    
    protected function verifySgEstado(&$row, $rowNum, $columnName = 'SGESTADO')
    {
        if(!is_null($row[$columnName])) {
            
            if (!ctype_alpha(strval($row[$columnName]))) {
                self::setError($rowNum, $columnName, $row[$columnName], "ONLY_CHAR");
            }
            
            if (strlen($row[$columnName]) > 2) {
                self::setError($rowNum, $columnName, $row[$columnName], "2_NUMBERS_EXECEDED");
            }
            
            $row[$columnName] = self::formatColumn($row[$columnName], 2);
        }
    }
    
    protected function verifyCdMunicipio(&$row, $rowNum, $columnName = 'CDMUNICIPIO')
    {
        if(!is_null($row[$columnName])) {
            
            if (!ctype_digit(strval($row[$columnName]))) {
                self::setError($rowNum, $columnName, $row[$columnName], "ONLY_ACCEPTS_NUMBERS");
            }
            
            if (strlen($row[$columnName]) > 4) {
                self::setError($rowNum, $columnName, $row[$columnName], "4_NUMBERS_EXECEDED");
            }
            
            $row[$columnName] = self::formatColumn($row[$columnName], 4);
        }
    }
    
    protected function formatColumn($column, $columnSize, $isNumber = false) {
        return $isNumber ? str_pad(preg_replace( '/[^0-9]/is', '', $column ), $columnSize, '0', STR_PAD_LEFT) 
                         : str_pad($column, $columnSize, '0', STR_PAD_LEFT);
    }
    
    protected function verifyCdOperador(&$row, $rowNum, $columnName)
    {
        if(!is_null($row[$columnName])) {
            $row[$columnName] = self::formatColumn($row[$columnName], 12);
            
            if (strlen($row[$columnName]) > 12) {
                self::setError($rowNum, $columnName, $row[$columnName], "12_CHAR_EXECEDED");
            }
            
            if (!$this->service->verifyCdOperadorExists($row, $columnName)) {
                self::setError($rowNum, $columnName, $row[$columnName], "UNREGISTERED");
            }
        }
    }
    
    protected function verifyExistErros($data)
    {
        foreach ($data as $row) {
            if (!empty($row['ERROS']))
                throw new \Exception('THERE_ARE_ERROR');
        } 
    }
    
    public function formatToView(&$data)
    {
        foreach ($data as &$row) {
           $row['IDERROR'] = !empty($row['ERROS']) ? 'S' : 'N';
        }
    }
    
    public function formatToSave(&$data)
    {
        foreach ($data as &$row) {
            $row['__is_new'] = true;
            $row['DTINCLUSAO']      = NULL;
            $row['DTULTATU']        = NULL;
            $row['NRORG']           = $this->environment->getNrorg();
            $row['NRORGINCLUSAO']   = $this->environment->getNrorg();
            $row['NRORGULTATU']     = $this->environment->getNrorg();
            $row['CDOPERINCLUSAO']  = $this->environment->getCdOperador();
            $row['CDOPERULTATU']    = $this->environment->getCdOperador();
            $row['IDATIVO']         = 'S';
        } 
    }
}